#
# pashto.py <Peter.Bienstman@UGent.be>
#

from mnemosyne.libmnemosyne.gui_translator import _
from mnemosyne.libmnemosyne.language import Language


class Pashto(Language):

    name = _("Pashto")
    used_for = "ps"
    feature_description = _("Google translation.")
