#
# kurdish.py <Peter.Bienstman@UGent.be>
#

from mnemosyne.libmnemosyne.gui_translator import _
from mnemosyne.libmnemosyne.language import Language


class Kurdish(Language):

    name = _("Kurdish")
    used_for = "ku"
    feature_description = _("Google translation.")
