//
// dlgs.h <Peter.Bienstman@UGent.be>
//

void add_cards_dlg_activate();

void edit_card_dlg_activate(char* card_id, int allow_cancel);

void activate_cards_dlg_activate();

void browse_cards_dlg_activate();

void card_appearance_dlg_activate();

void activate_plugins_dlg_activate();

void manage_card_types_dlg_activate();

void statistics_dlg_activate();

void configuration_dlg_activate();
    
void sync_dlg_activate();
