#
# somali.py <Peter.Bienstman@UGent.be>
#

from mnemosyne.libmnemosyne.gui_translator import _
from mnemosyne.libmnemosyne.language import Language


class Somali(Language):

    name = _("Somali")
    used_for = "so"
    feature_description = _("Google translation.")
