#
# hawaiian.py <Peter.Bienstman@UGent.be>
#

from mnemosyne.libmnemosyne.gui_translator import _
from mnemosyne.libmnemosyne.language import Language


class Hawaiian(Language):

    name = _("Hawaiian")
    used_for = "haw"
    feature_description = _("Google translation.")
